@@ -89,6 +89,7 @@
 #define I450T       0x4641
 #define I550        0x4642
 #define I200        0x4646
+#define SAGE        0x4647
 #define I300C       0x4648
 #define I200C       0x4649
 #define I100        0x464E
@@ -191,7 +192,7 @@ oceanic_atom2_parser_create (dc_parser_t **out, dc_context_t *context, unsigned
 		parser->footersize = 0;
 	} else if (model == A300CS || model == VTX ||
 		model == I450T || model == I750TC ||
-		model == I770R) {
+		model == I770R || model == SAGE) {
 		parser->headersize = 5 * PAGESIZE;
 	} else if (model == PROPLUSX) {
 		parser->headersize = 3 * PAGESIZE;
@@ -345,6 +346,7 @@ oceanic_atom2_parser_get_datetime (dc_parser_t *abstract, dc_datetime_t *datetim
 		case I750TC:
 		case PROPLUSX:
 		case I770R:
+		case SAGE:
 			datetime->year   = (p[10]) + 2000;
 			datetime->month  = (p[8]);
 			datetime->day    = (p[9]);
@@ -466,7 +468,7 @@ oceanic_atom2_parser_cache (oceanic_atom2_parser_t *parser)
 		he_offset = 0x48;
 		ngasmixes = 6;
 	} else if (parser->model == A300CS || parser->model == VTX ||
-		parser->model == I750TC) {
+		parser->model == I750TC || parser->model == SAGE) {
 		o2_offset = 0x2A;
 		if (data[0x39] & 0x04) {
 			ngasmixes = 1;
@@ -530,9 +532,7 @@ oceanic_atom2_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, uns
 {
 	dc_status_t status = DC_STATUS_SUCCESS;
 	oceanic_atom2_parser_t *parser = (oceanic_atom2_parser_t *) abstract;
-
 	const unsigned char *data = abstract->data;
-	unsigned int size = abstract->size;
 
 	// Cache the header data.
 	status = oceanic_atom2_parser_cache (parser);
@@ -689,7 +689,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		unsigned int idx = 0x17;
 		if (parser->model == A300CS || parser->model == VTX ||
 			parser->model == I450T || parser->model == I750TC ||
-			parser->model == PROPLUSX || parser->model == I770R)
+			parser->model == PROPLUSX || parser->model == I770R ||
+			parser->model == SAGE)
 			idx = 0x1f;
 		switch (data[idx] & 0x03) {
 		case 0:
@@ -745,7 +746,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 		parser->model == TX1 || parser->model == A300CS ||
 		parser->model == VTX || parser->model == I450T ||
 		parser->model == I750TC || parser->model == PROPLUSX ||
-		parser->model == I770R || parser->model == I470TC) {
+		parser->model == I770R || parser->model == I470TC ||
+		parser->model == SAGE) {
 		samplesize = PAGESIZE;
 	}
 
@@ -830,7 +832,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 				tank = 0;
 				pressure = (((data[offset + 7] << 8) + data[offset + 6]) & 0x0FFF);
 			} else if (parser->model == A300CS || parser->model == VTX ||
-				parser->model == I750TC || parser->model == I770R) {
+				parser->model == I750TC || parser->model == I770R ||
+				parser->model == SAGE) {
 				// Tank pressure (1 psi) and number (one based index)
 				tank = (data[offset + 1] & 0x03) - 1;
 				pressure = ((data[offset + 7] << 8) + data[offset + 6]) & 0x0FFF;
@@ -932,7 +935,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					temperature = ((data[offset + 7] & 0xF0) >> 4) | ((data[offset + 7] & 0x0C) << 2) | ((data[offset + 5] & 0x0C) << 4);
 				} else if (parser->model == A300CS || parser->model == VTX ||
 					parser->model == I750TC || parser->model == PROPLUSX ||
-					parser->model == I770R) {
+					parser->model == I770R|| parser->model == SAGE) {
 					temperature = data[offset + 11];
 				} else {
 					unsigned int sign;
@@ -976,7 +979,8 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 					pressure = (((data[offset + 0] & 0x03) << 8) + data[offset + 1]) * 5;
 				else if (parser->model == TX1 || parser->model == A300CS ||
 					parser->model == VTX || parser->model == I750TC ||
-					parser->model == PROPLUSX || parser->model == I770R)
+					parser->model == PROPLUSX || parser->model == I770R ||
+					parser->model == SAGE)
 					pressure = array_uint16_le (data + offset + 4);
 				else
 					pressure -= data[offset + 1];
@@ -1027,7 +1031,7 @@ oceanic_atom2_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_
 			unsigned int have_deco = 0;
 			unsigned int decostop = 0, decotime = 0;
 			if (parser->model == A300CS || parser->model == VTX ||
-				parser->model == I750TC ||
+				parser->model == I750TC || parser->model == SAGE ||
 				parser->model == PROPLUSX || parser->model == I770R) {
 				decostop = (data[offset + 15] & 0x70) >> 4;
 				decotime = array_uint16_le(data + offset + 6) & 0x03FF;
