@@ -21,6 +21,7 @@
 
 #include <stdlib.h>
 
+#define WIN32_LEAN_AND_MEAN
 #define NOGDI
 #include <windows.h>
 
@@ -57,7 +58,7 @@ struct dc_serial_device_t {
 
 typedef struct dc_serial_iterator_t {
 	dc_iterator_t base;
-	dc_filter_t filter;
+	dc_descriptor_t *descriptor;
 	HKEY hKey;
 	DWORD count;
 	DWORD current;
@@ -180,7 +181,7 @@ dc_serial_iterator_new (dc_iterator_t **out, dc_context_t *context, dc_descripto
 		}
 	}
 
-	iterator->filter = dc_descriptor_get_filter (descriptor);
+	iterator->descriptor = descriptor;
 	iterator->hKey = hKey;
 	iterator->count = count;
 	iterator->current = 0;
@@ -226,7 +227,7 @@ dc_serial_iterator_next (dc_iterator_t *abstract, void *out)
 		// Null terminate the string.
 		data[data_len] = 0;
 
-		if (iterator->filter && !iterator->filter (DC_TRANSPORT_SERIAL, data)) {
+		if (!dc_descriptor_filter (iterator->descriptor, DC_TRANSPORT_SERIAL, data, NULL)) {
 			continue;
 		}
 
