@@ -244,6 +244,9 @@ shearwater_petrel_device_foreach (dc_device_t *abstract, dc_dive_callback_t call
 	case 0x1F0A:
 		model = TERIC;
 		break;
+	case 0x1512:
+		model = PEREGRINE;
+		break;
 	default:
 		// return a model of 0 which is unknown
 		WARNING (abstract->context, "Unknown hardware type %04x. Assuming Petrel.", hardware);
