@@ -352,9 +352,7 @@ static dc_status_t
 suunto_d9_parser_get_field (dc_parser_t *abstract, dc_field_type_t type, unsigned int flags, void *value)
 {
 	suunto_d9_parser_t *parser = (suunto_d9_parser_t*) abstract;
-
 	const unsigned char *data = abstract->data;
-	unsigned int size = abstract->size;
 
 	// Cache the gas mix data.
 	dc_status_t rc = suunto_d9_parser_cache (parser);
@@ -571,7 +569,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 		if ((nsamples + 1) == marker) {
 			while (offset < size) {
 				unsigned int event = data[offset++];
-				unsigned int seconds, type, unknown, heading;
+				unsigned int seconds, type, state, number, heading;
 				unsigned int current, next;
 				unsigned int he, o2, ppo2, idx;
 				unsigned int length;
@@ -600,7 +598,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
 					}
-					unknown = data[offset + 0];
+					state   = data[offset + 0];
 					seconds = data[offset + 1];
 					sample.event.type = SAMPLE_EVENT_SURFACE;
 					sample.event.time = seconds;
@@ -719,7 +717,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 						ERROR (abstract->context, "Buffer overflow detected!");
 						return DC_STATUS_DATAFORMAT;
 					}
-					unknown = data[offset + 0];
+					number  = data[offset + 0];
 					seconds = data[offset + 1];
 					heading = array_uint16_le (data + offset + 2);
 					if (heading == 0xFFFF) {
@@ -775,7 +773,7 @@ suunto_d9_parser_samples_foreach (dc_parser_t *abstract, dc_sample_callback_t ca
 					if ((type & 0x80) == 0) {
 						idx += parser->nccr;
 					}
-					if (idx >= parser->ngasmixes) {
+					if (idx >= parser->ngasmixes || o2 != parser->oxygen[idx] || he != parser->helium[idx]) {
 						ERROR (abstract->context, "Invalid gas mix.");
 						return DC_STATUS_DATAFORMAT;
 					}
